#!/usr/bin/env python3

from sys import platform, argv, exit

import pyforms_lite

from .forms import *
from .updateTBA import main as utmain
from .playlistToTBA import main as pttmain
from .updatePlaylistThumbnails import main as uptmain


def main():
    if len(argv) > 1:
        if "-p" in argv:
            pttmain()
            exit(0)
        elif "-u" in argv:
            utmain()
            exit(0)
        elif "-t" in argv:
            uptmain()
            exit(0)
        else:
            print("Not a valid option")
            print("Valid options include [-p|-u|-t]")
            print("-p will load playlistToTBA")
            print("-u will load updateTBA")
            print("-t will load updatePlaylistThumbnails")
            exit(0)
    try:
        pyforms_lite.start_app(FRC_Uploader, geometry=(200, 200, 1, 1))
    except Exception as e:
        print(e)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from googleapiclient.errors import HttpError
from oauth2client.tools import argparser
from .youtube import get_youtube_service
from googleapiclient.http import MediaFileUpload

THUMBNAIL = ""
PLAYLISTID = ""


def update_thumbnail(youtube, video_id, thumbnail):
    youtube.thumbnails().set(
        videoId=video_id,
        media_body=thumbnail).execute()
    print(f"Thumbnail added to video {video_id}")


def update_thumbnails(youtube, pID, thumbnail):
    playlistitems_list = youtube.playlistItems().list(
        playlistId=pID, part="snippet", maxResults=50).execute()
    print(playlistitems_list)
    try:
        nextPageToken = playlistitems_list["nextPageToken"]
    except KeyError as e:
        print("Only one page")
    while ('nextPageToken' in playlistitems_list):
        print("getting next page")
        nextPageList = youtube.playlistItems().list(
            playlistId=pID,
            part="snippet",
            maxResults=50,
            pageToken=nextPageToken).execute()
        print("got next page")
        playlistitems_list["items"] = playlistitems_list["items"] + \
            nextPageList["items"]
        if "nextPageToken" not in nextPageList:
            playlistitems_list.pop('nextPageToken', None)
            print("no more pages")
        else:
            nextPageToken = nextPageList['nextPageToken']
            print("got next page token")
            # Print information about each video.
    errorvids = []
    for playlist_item in playlistitems_list["items"]:
        title = playlist_item["snippet"]["title"]
        video_id = playlist_item["snippet"]["resourceId"]["videoId"]
        try:
            update_thumbnail(youtube, video_id, thumbnail)
        except HttpError as e:
            x = (title, video_id)
            errorvids.append(x)
            continue
        print("thumbnail updated")

    for tup in errorvids:
        print(tup)


def main():
    youtube = get_youtube_service()
    PID = input("Playlist Link: ")
    THUMBNAIL = input("Thumbnail file name: ")
    f = PID.find("PL")
    PID = PID[f:f+34]
    try:
        update_thumbnails(youtube, PID, THUMBNAIL)
    except HttpError as e:
        print(f"An HTTP error {e.resp.status} occurred:\n{e.content}")



#!/usr/bin/env python3
""" this script will take any playlist that uploaded videos using the uploader and link the matches to TBA"""

import json
from .youtube import *
from .consts import DEFAULT_DESCRIPTION, CREDITS
from .utils import post_video, quarters_match_code, semis_match_code, finals_match_code, tiebreak_mnum, get_match_results


def update_description(youtube, snippet, vID, ecode, mcode, ename, team, twit, fb, weblink):
    """
    Creates an updated TBA Description and updates the video
    """
    description = DEFAULT_DESCRIPTION + CREDITS
    print(snippet)
    blue_data, red_data = get_match_results(ecode, mcode)
    description = description.format(ecode=ecode, ename=ename, team=team, twit=twit, fb=fb, weblink=weblink, red1=red_data[1], red2=red_data[2], red3=red_data[3], redscore=red_data[0], blue1=blue_data[1], blue2=blue_data[2], blue3=blue_data[3], bluescore=blue_data[0])
    snippet['snippet']['description'] = description
    snippet['snippet']['categoryId'] = 28
    youtube.videos().update(
        part='snippet',
        body=dict(
            snippet=snippet['snippet'],
            id=vID)
    ).execute()
    print(f"Updated description of {vID}")


def main():
    PID = input("Link to Playlist: ")
    f = PID.find("PL")
    PID = PID[f:f + 34]
    TBAID = input("TBA ID: ")
    TBASECRET = input("TBA Secret: ")
    ecode = input("Event Code (eg: 2018incmp): ")
    ename = input("Event Name: ")
    team = input("Production Team Name: ")
    twit = input("Twitter Handle: ")
    fb = input("Facebook Name: ")
    weblink = input("Website Link")

    if (TBAID == "" or TBASECRET == ""):
        print("Can't add to TBA without ID and Secret")
        sys.exit(0)

    youtube = get_youtube_service()

    playlistitems_list = youtube.playlistItems().list(
        playlistId=PID,
        part="snippet",
        maxResults=50
    ).execute()

    print("got list")
    nextPageToken = playlistitems_list["nextPageToken"]
    while ('nextPageToken' in playlistitems_list):
        num = None
        print("getting next page")
        nextPageList = youtube.playlistItems().list(
            playlistId=PID,
            part="snippet",
            maxResults=50,
            pageToken=nextPageToken).execute()
        print("got next page")
        playlistitems_list["items"] = playlistitems_list["items"] + \
            nextPageList["items"]
        if "nextPageToken" not in nextPageList:
            playlistitems_list.pop('nextPageToken', None)
            print("no more pages")
        else:
            nextPageToken = nextPageList['nextPageToken']
            print("got next page token")
            # Print information about each video.

    for playlist_item in playlistitems_list["items"]:
        title = str(playlist_item["snippet"]["title"])
        video_id = str(playlist_item["snippet"]["resourceId"]["videoId"])
        if "Qual" in title:
            mnum = None
            try:
                mnum = f"qm{title[title.find('Match') + 5:].split(' ')[1]}"
            except IndexError as e:
                print(title)
                print(e)
            body = json.dumps({mnum: video_id})
            print(f"Posting {mnum}")
            post_video(TBAID, TBASECRET, body, ecode, "match_videos")
            update_description(youtube, playlist_item, video_id, ecode, mnum, ename, team, twit, fb, weblink)
        elif "Quarterfinal" in title:
            try:
                num = int(title[title.find("Match") + 5:].split(" ")[1])
            except ValueError as e:
                num = int(title[title.find("Tiebreaker") + 10:].split(" ")[1])
            if "Tiebreak" in title:
                num = tiebreak_mnum(num, "qf")
            mnum = quarters_match_code("qf", num)
            body = json.dumps({mnum: video_id})
            print(f"Posting {mnum}")
            post_video(TBAID, TBASECRET, body, ecode, "match_videos")
            update_description(youtube, playlist_item, video_id, ecode, mnum, ename, team, twit, fb, weblink)
        elif "Semifinal" in title:
            try:
                num = int(title[title.find("Match") + 5:].split(" ")[1])
            except ValueError as e:
                num = int(title[title.find("Tiebreaker") + 10:].split(" ")[1])
            if "Tiebreak" in title:
                num = tiebreak_mnum(num, "sf")
            mnum = semis_match_code("sf", num)
            body = json.dumps({mnum: video_id})
            print(f"Posting {mnum}")
            post_video(TBAID, TBASECRET, body, ecode, "match_videos")
            update_description(youtube, playlist_item, video_id, ecode, mnum, ename, team, twit, fb, weblink)
        elif "Final" in title:
            try:
                num = int(title[title.find("Match") + 5:].split(" ")[1])
            except ValueError as e:
                num = int(title[title.find("Tiebreaker") + 10:].split(" ")[1])
            if "Tiebreak" in title:
                num = tiebreak_mnum(num, "f1m")
            mnum = finals_match_code("f1m", num)
            body = json.dumps({mnum: video_id})
            print(f"Posting {mnum}")
            post_video(TBAID, TBASECRET, body, ecode, "match_videos")
            update_description(youtube, playlist_item, video_id, ecode, mnum, ename, team, twit, fb, weblink)
        elif any(k in title for k in ("Opening", "Closing", "Awards", "Alliance", "Highlight")):
            body = json.dumps([video_id])
            print(f"Posting {title}")
            post_video(TBAID, TBASECRET, body, ecode, "media")
        else:
            print("I don't know what this is")
            print(title)
            print(video_id)



#!/usr/bin/env python3
import pkg_resources

from .youtube import *

import tbapy

__version__ = pkg_resources.require("FRCUploader")[0].version

# Default Variables
DEBUG = False  # DON'T COMMIT THIS LINE IF TRUE
DEFAULT_TAGS = "{}, frcuploader, FIRST, omgrobots, FRC, FIRST Robotics Competition, robots, Robotics, {game}"
MATCH_TYPE = ("qm", "qf", "sf", "f1m")
DEFAULT_DESCRIPTION = """Footage of the {ename} is courtesy of {team}.

Red Alliance  ({red1}, {red2}, {red3}) - {redscore}
Blue Alliance ({blue3}, {blue2}, {blue1}) - {bluescore}

To view match schedules and results for this event, visit The Blue Alliance Event Page: https://www.thebluealliance.com/event/{ecode}

Follow us on Twitter (@{twit}) and Facebook ({fb}).

For more information and future event schedules, visit our website: {weblink}

Thanks for watching!"""

NO_TBA_DESCRIPTION = """Footage of the {ename} Event is courtesy of {team}.

Follow us on Twitter (@{twit}) and Facebook ({fb}).

For more information and future event schedules, visit our website: {weblink}

Thanks for watching!"""

CREDITS = """

Uploaded with FRC-YouTube-Uploader (https://github.com/NikhilNarayana/FRC-YouTube-Uploader) by Nikhil Narayana"""

VALID_PRIVACY_STATUSES = ("public", "unlisted", "private")

GAMES = {
    "2019": "FIRST Destination: Deep Space, Destination: Deep Space, Deep Space",
    "2018": "FIRST Power Up, FIRST POWER UP",
    "2017": "FIRST Steamworks, FIRST STEAMworks",
    "2016": "FIRST Stronghold",
    "2015": "Recycle Rush",
    "2014": "Aerial Assist",
    "2013": "Ultimate Ascent"
}

spreadsheetID = "18flsXvAcYvQximmeyG0-9lhYtb5jd_oRtKzIN7zQDqk"
rowRange = "Data!A1:G1"
firstrun = True
stop_thread = False
response = None
status = None
error = None
sleep_minutes = 600
retry = 0
retryforlimit = 0
retry_status_codes = get_retry_status_codes()
retry_exceptions = get_retry_exceptions()
max_retries = get_max_retries()
tags = None
mcode = None
youtube = get_youtube_service()
spreadsheet = get_spreadsheet_service()
tba = tbapy.TBA("wvIxtt5Qvbr2qJtqW7ZsZ4vNppolYy0zMNQduH8LdYA7v2o1myt8ZbEOHAwzRuqf")
sizes = ("bytes", "KB", "MB", "GB", "TB")
cerem = ("None", "Opening Ceremonies", "Alliance Selection", "Closing Ceremonies", "Highlight Reel")
queue_values = os.path.join(os.path.expanduser("~"), ".frc_queue_values.txt")
form_values = os.path.join(os.path.expanduser("~"), '.frc_form_values.json')
log_file = os.path.join(os.path.expanduser("~"), ".frc_log.txt")
rec_formats = (".ts", ".mkv", ".avi", ".mp4", ".flv", ".mov", ".flv")



#!/usr/bin/env python3
"""This is a backup solution for if the main script doesn't update TBA properly"""

import json
from urllib.parse import *

from .utils import post_video


def video_id(value):
    query = urlparse(value)
    if query.hostname == 'youtu.be':
        return query.path[1:]
    if query.hostname in ('www.youtube.com', 'youtube.com'):
        if query.path == '/watch':
            p = parse_qs(query.query)
            return p['v'][0]
        if query.path[:7] == '/embed/':
            return query.path.split('/')[2]
        if query.path[:3] == '/v/':
            return query.path.split('/')[2]
    return value


def main():
    loc = "match_video"
    request_body = None
    codes = None
    vID = video_id(input("Video Link: "))
    link = input("Blue Alliance Match Link (eg.https://www.thebluealliance.com/match/2017gaalb_sf1m1) : ")
    if "media" in link:
        loc = "media"
        request_body = json.dumps([vID])
    else:
        codes = link.split("/")[-1].split("_")
        request_body = json.dumps({codes[-1]: vID})
    tbaID = input("TBA ID: ")
    tbaSecret = input("TBA Secret: ")
    post_video(tbaID, tbaSecret, request_body, codes[0], loc)



#!/usr/bin/env python3

import os
import sys
import json
import pickle
import threading
import subprocess
from time import sleep
from queue import Queue
from copy import deepcopy

from . import consts
from . import utils

import requests
from datetime import datetime
from argparse import Namespace

from pyforms_lite import BaseWidget
from PyQt5 import QtCore, QtGui
from pyforms_lite.controls import ControlNumber
from pyforms_lite.controls import ControlText, ControlDir
from pyforms_lite.controls import ControlTextArea, ControlList
from pyforms_lite.controls import ControlCombo, ControlProgress
from pyforms_lite.controls import ControlButton, ControlCheckBox


class EmittingStream(QtCore.QObject):
    """
    Capture any uses of print so it can be printed to a custom text box
    """
    textWritten = QtCore.pyqtSignal(str)

    def write(self, text):
        self.textWritten.emit(str(text))

    def flush(self):
        pass


class FRC_Uploader(BaseWidget):
    """
    GUI constructor.
    Create the GUI using pyforms_lite.start_app(FRC_Uploader)
    """

    def __init__(self):
        try:  # check if the user can update the app
            latest_version = requests.get('https://pypi.org/pypi/FRCUploader/json').json()['info']['version']
            if any(int(x) > int(y) for x, y in zip(latest_version.split("."), consts.__version__.split("."))):  # prevents messages when developing
                if "linux" in sys.platform:
                    self.message(f"Current Version: {consts.__version__}\nVersion {latest_version} is available.\n You can update with this command: pip3 install -U frcuploader=={latest_version}", title="FRCUploader")
                else:
                    resp = self.question(f"Current Version: {consts.__version__}\nVersion {latest_version} is available. Would you like to update?", title="FRCUploader")
                    if resp == "yes":
                        subprocess.call(('pip3', 'install', '-U', f'frcuploader=={latest_version}'))
                        self.message("You can now restart the app to use the new version", title="FRCUploader")
        except Exception as e:
            print(e)
        super(FRC_Uploader, self).__init__("FRC YouTube Uploader")

        # Redirct print output
        sys.stdout = EmittingStream(textWritten=self.write_print)

        # Queue
        self._queue = Queue()
        self._queueref = []

        # Redirect error output to status window and a file
        sys.stderr = EmittingStream(textWritten=self.write_err)

        # Create form fields
        # Event Values
        self._where = ControlDir(" Match Files Location")
        self._newest = ControlCheckBox("Get Newest File")
        self._sendto = ControlDir(" Move Files To")
        self._prodteam = ControlText(" Production Team")
        self._twit = ControlText("Twitter Handle")
        self._fb = ControlText("Facebook Name")
        self._weblink = ControlText(" Website Link")
        self._ename = ControlText("Event Name")
        self._ecode = ControlText("Event Code")
        self._pID = ControlText(" Playlist ID")
        self._tbaID = ControlText("TBA ID")
        self._tbaSecret = ControlText("TBA Secret")
        self._privacy = ControlCombo("Video Privacy Status")
        self._description = ControlTextArea(" Video Description")
        self._description.add_popup_menu_option("Reset", self.__reset_descrip_event)

        # Match Values
        self._mcode = ControlText("Match Code", visible=False, helptext="READ THE INSTRUCTIONS TO FIND OUT HOW TO USE THIS!")
        self._mnum = ControlNumber("Match Number", minimum=1, maximum=500)
        self._mtype = ControlCombo("Match Type")
        self._tiebreak = ControlCheckBox("Tiebreaker")
        self._tba = ControlCheckBox("Use TBA")
        self._replay = ControlCheckBox("Replay")
        self._ceremonies = ControlCombo("Ceremonies")
        self._eday = ControlCombo("Event Day")
        self._end = ControlNumber("Last Match Number", minimum=0, maximum=500)

        # Output Box
        self._output = ControlTextArea()
        self._output.readonly = True
        self._qview = ControlList("Queue", select_entire_row=True)
        self._qview.cell_double_clicked_event = self.__ignore_job
        self._qview.readonly = True
        self._qview.horizontal_headers = ["Event Code", "Match Type", "Match #"]

        # Button
        self._button = ControlButton('Submit')
        self._ascrollbutton = ControlButton("Toggle Scroll")
        self._autoscroll = True

        # Form Layout
        self.formset = [{
            "-Match Values":
            [(' ', "_mcode", ' '), (' ', "_mnum", ' '), (' ', "_mtype", ' '),
             (' ', "_tiebreak", "_tba", "_replay", ' '), (' ', "_ceremonies", ' '),
             (' ', "_eday", ' '), (' ', "_end", ' ')],
            "-Status Output-":
            ["_output", (' ', "_ascrollbutton", ' '), "=", "_qview"],
            "Event Values-": [("_where", "_newest"), "_sendto", ("_prodteam", "_twit", "_fb"),
                              ("_weblink", "_ename", "_ecode"),
                              ("_pID", "_tbaID", "_tbaSecret"), ("_privacy", " "), "_description"]
        }, (' ', '_button', ' ')]

        # Main Menu Layout
        self.mainmenu = [{
            'Settings': [{'Reset Form Values': self.__reset_form_event}, {'Youtube Log Out': self.__reset_cred_event}, {'Show/Hide Match Code': self.__toggle_match_code}],
            'Queue': [{'Toggle Uploads': self.__toggle_worker}, {'Save Queue': self.__save_queue}, {'Load Queue': self.__load_queue}]}]

        # Set TBA check
        self._tba.value = True

        # Set Default Text
        self._tbaID.value = "Go to thebluealliance.com/request/apiwrite to get keys"
        self._tbaSecret.value = "Go to thebluealliance.com/request/apiwrite to get keys"
        self._description.value = consts.DEFAULT_DESCRIPTION
        self._mcode.value = "0"
        self._mnum.value = 1

        # Add ControlCombo values
        for t in consts.VALID_PRIVACY_STATUSES:
            self._privacy += t
        self._mtype += ("Qualifications", "qm")
        self._mtype += ("Quarterfinals", "qf")
        self._mtype += ("Semifinals", "sf")
        self._mtype += ("Finals", "f1m")
        self._ceremonies += ("None", 0)
        self._ceremonies += ("Opening Ceremonies", 1)
        self._ceremonies += ("Alliance Selection", 2)
        self._ceremonies += ("Closing Ceremonies", 3)
        self._ceremonies += ("Highlight Reel", 4)
        self._eday += ("Ignore", 0)
        self._eday += ("1", 1)
        self._eday += ("2", 2)
        self._eday += ("3", 3)

        # Define the button action
        self._button.value = self.__button_action
        self._ascrollbutton.value = self.__togglescroll

        self.testval = 0

        # Get latest values from frc_form_values.json
        try:
            with open(consts.form_values) as f:
                values = json.load(f)
                i = 0
                switcher = {
                    0: self._where,
                    1: self._prodteam,
                    2: self._twit,
                    3: self._fb,
                    4: self._weblink,
                    5: self._ename,
                    6: self._ecode,
                    7: self._pID,
                    8: self._tbaID,
                    9: self._tbaSecret,
                    10: self._description,
                    11: self._mcode,
                    12: self._mnum,
                    13: self._mtype,
                    14: self._tiebreak,
                    15: self._tba,
                    16: self._ceremonies,
                    17: self._eday,
                    18: self._end,
                    19: self._newest,
                    20: self._privacy,
                    21: self._sendto,
                }
                for val in values:
                    if i > 19:
                        break
                    if val is not "":
                        if any(i == k for k in (14, 15, 19)):
                            if val == "no":
                                switcher[i].value = False
                            else:
                                switcher[i].value = True
                        elif i == 12 or i == 18:
                            try:
                                switcher[i].value = int(val)
                            except Exception as e:
                                pass
                        else:
                            switcher[i].value = val
                    i = i + 1
        except (IOError, OSError, StopIteration) as e:
            print("No frc_form_values.json to read from, continuing with default values and creating file")
            with open(consts.form_values, "w+") as f:  # if the file doesn't exist
                f.write(json.dumps([]))

    def __togglescroll(self):
        self._autoscroll = False if self._autoscroll else True

    def __button_action(self):
        """Manipulates and transforms data from the forms into usable
           data that can be used for uploading videos"""
        options = Namespace()
        row = [0] * 22
        options.where = row[0] = self._where.value
        options.prodteam = row[1] = self._prodteam.value
        options.twit = row[2] = self._twit.value
        options.fb = row[3] = self._fb.value
        options.weblink = row[4] = self._weblink.value
        options.ename = row[5] = self._ename.value
        options.ecode = row[6] = self._ecode.value
        f = self._pID.value.find("PL")
        self._pID.value = self._pID.value[f:f + 34]
        options.pID = row[7] = self._pID.value
        options.tbaID = row[8] = self._tbaID.value
        options.tbaSecret = row[9] = self._tbaSecret.value
        options.description = row[10] = self._description.value
        options.mcode = row[11] = self._mcode.value
        options.mnum = row[12] = int(self._mnum.value)
        options.mtype = row[13] = self._mtype.value
        options.tiebreak, row[14] = (True, "yes") if self._tiebreak.value else (False, "no")
        options.tba, row[15] = (True, "yes") if self._tba.value else (False, "no")
        options.ceremonies = row[16] = self._ceremonies.value
        options.eday = row[17] = self._eday.value
        options.end = row[18] = 0
        options.replay = self._replay.value
        self._replay.value = False
        options.newest, row[19] = (True, "yes") if self._newest.value else (False, "no")
        if options.newest:
            files = list(reversed([f for f in os.listdir(options.where) if os.path.isfile(os.path.join(options.where, f)) and not f.startswith('.') and any(f.endswith(z) for z in consts.rec_formats)]))
            options.file = max([os.path.join(options.where, f) for f in files], key=os.path.getmtime)
        options.privacy = row[20] = self._privacy.value
        options.sendto = row[21] = self._sendto.value
        options.ignore = False
        if not int(self._end.value):
            if options.ceremonies:
                self._qview += (options.ecode, consts.cerem[options.ceremonies], "N/A")
            else:
                self._qview += (options.ecode, options.mtype, options.mnum)
        elif not options.newest:
            for r in range(options.mnum, int(self._end.value)):
                self._qview += (options.ecode, options.mtype, options.mnum)
                self._queue.put(options)
                self._queueref.append(options)
                self._qview.resize_rows_contents()
                options = deepcopy(options)
                options.mnum += 1
        else:
            print("Using Last Match Number and Get Newest File together is not supported")
            print(f"Will fallback to just uploading the newest file for mnum {options.mnum}")
            self._end.value = 0
        if int(self._end.value):
            self._qview += (options.ecode, options.mtype, options.mnum)
        self._queue.put(options)
        self._queueref.append(options)
        self._qview.resize_rows_contents()
        if consts.firstrun:
            thr = threading.Thread(target=self.__worker)
            thr.daemon = True
            thr.start()
            consts.firstrun = False
        if not self._ceremonies.value:
            if not int(self._end.value):
                self._mnum.value = self._mnum.value + 1
            else:
                self._mnum.value = self._end.value + 1
                self._end.value = 0
        elif self._ceremonies.value == 2:
            self._mnum.value = 1
            self._mtype.value = "qf"
        if self._mtype.value == "qm" and self._tiebreak.value:
            row[14] = self._tiebreak.value = False
        row[12] = int(self._mnum.value)
        row[18] = int(self._end.value)
        with open(consts.form_values, 'w') as f:
            f.write(json.dumps(row))

    def write_print(self, text):
        self._output._form.plainTextEdit.insertPlainText(text)
        if self._autoscroll:
            self._output._form.plainTextEdit.moveCursor(QtGui.QTextCursor.End)
        print(text, file=sys.__stdout__, end='')

    def write_err(self, text):
        self._output._form.plainTextEdit.insertPlainText(text)
        if self._autoscroll:
            self._output._form.plainTextEdit.moveCursor(QtGui.QTextCursor.End)
        print(text, file=sys.__stdout__, end='')
        with open(consts.log_file, "a") as f:
            f.write(text)

    def __worker(self):
        while True:
            options = self._queue.get()
            if not options.ignore:
                options.then = datetime.now()
                utils.init(options)
                self._qview -= 0
                self._queueref.pop(0)
            self._queue.task_done()

    def __toggle_worker(self):
        if not consts.stop_thread:
            print("Stopping Uploads")
            consts.stop_thread = True
            consts.firstrun = False
        else:
            print("Ready to Upload")
            consts.stop_thread = False
            consts.firstrun = True

    def __save_queue(self):
        with open(consts.queue_values, "wb") as f:
            f.write(pickle.dumps(self._queueref))

    def __load_queue(self):
        try:
            with open(consts.queue_values, "rb") as f:
                self._queueref = pickle.load(f)
        except Exception as e:
            print("You need to save a queue before loading a queue")
            return
        for options in self._queueref:
            if options.ceremonies:
                self._qview += (options.ecode, consts.cerem[options.ceremonies], "N/A")
            else:
                self._qview += (options.ecode, options.mtype, options.mnum)
            self._queue.put(options)
            self._qview.resize_rows_contents()
        thr = threading.Thread(target=self.__worker)
        thr.daemon = True
        consts.firstrun = False
        consts.stop_thread = False
        thr.start()

    def __reset_form_event(self):
        with open(consts.form_values, "w+") as f:
            f.write(json.dumps([]))
        self._tbaID.value = "Go to thebluealliance.com/request/apiwrite to get keys"
        self._tbaSecret.value = "Go to thebluealliance.com/request/apiwrite to get keys"
        self._description.value = consts.DEFAULT_DESCRIPTION
        self._mcode.value = "0"
        self._mnum.value = 1
        self._end.value = 0
        self._mtype.value = "qm"
        self._ceremonies.value = 0
        self._eday.value = 0
        self._tba.value = True
        self._where.value = ""
        self._sendto.value = ""
        self._prodteam.value = ""
        self._twit.value = ""
        self._fb.value = ""
        self._weblink.value = ""
        self._ename.value = ""
        self._ecode.value = ""
        self._pID.value = ""

    def __reset_cred_event(self):
        title = consts.youtube.channels().list(part='snippet', mine=True).execute()
        title = title['items'][0]['snippet']['title']
        resp = self.question(f"You are currently logged into {title}\nWould you like to log out?", title="FRCUploader")
        if resp == "yes":
            os.remove(os.path.join(os.path.expanduser("~"), ".frc-oauth2-spreadsheet.json"))
            os.remove(os.path.join(os.path.expanduser("~"), ".frc-oauth2-youtube.json"))
            sys.exit(0)

    def __reset_descrip_event(self):
        self._description.value = consts.DEFAULT_DESCRIPTION

    def __ignore_job(self, row, column):
        self._qview -= row
        self._queueref[row].ignore = True
        self._queueref.pop(row)

    def __toggle_match_code(self):
        if self._mcode.visible:
            self._mcode.hide()
        else:
            self._mcode.show()



#!/usr/bin/env python3

try:
    import http.client as httplib
except ImportError:
    import httplib
import httplib2
import os
import sys
import errno
from decimal import Decimal

from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload

from oauth2client.file import Storage
from oauth2client.tools import run_flow
from oauth2client.client import flow_from_clientsecrets

httplib2.RETRIES = 1

RETRIABLE_EXCEPTIONS = (httplib2.HttpLib2Error, IOError, httplib.NotConnected,
                        httplib.IncompleteRead,
                        httplib.ImproperConnectionState,
                        httplib.CannotSendRequest, httplib.CannotSendHeader,
                        httplib.ResponseNotReady, httplib.BadStatusLine)

RETRIABLE_STATUS_CODES = [500, 502, 504]
YOUTUBE_API_SERVICE_NAME = "youtube"
YOUTUBE_API_VERSION = "v3"

YOUTUBE_UPLOAD_SCOPE = "https://www.googleapis.com/auth/youtube.upload https://www.googleapis.com/auth/youtube https://www.googleapis.com/auth/youtube.readonly https://www.googleapis.com/auth/youtube.force-ssl"
SPREADSHEETS_SCOPE = "https://www.googleapis.com/auth/spreadsheets"


def upload(yt, body, file):
    vid = None
    ret = None
    while not vid:
        insert_request = yt.videos().insert(
            part=",".join(body.keys()),
            body=body,
            media_body=MediaFileUpload(file,
                                       chunksize=104857600,
                                       resumable=True),)
        vid = upload_service(insert_request)
    return vid


def upload_service(insert_request):
        response = None
        retry_exceptions = get_retry_exceptions()
        retry_status_codes = get_retry_status_codes()
        ACCEPTABLE_ERRNO = (errno.EPIPE, errno.EINVAL, errno.ECONNRESET)
        try:
            ACCEPTABLE_ERRNO += (errno.WSAECONNABORTED,)
        except AttributeError:
            pass  # Not windows
        while True:
            try:
                status, response = insert_request.next_chunk()
                if status is not None:
                    percent = Decimal(int(status.resumable_progress) / int(status.total_size))
                    print(f"{round(100 * percent, 2)}% uploaded")
            except HttpError as e:
                if e.resp.status in retry_status_codes:
                    print(f"A retriable HTTP error {e.resp.status} occurred:\n{e.content}")
                elif b"503" in e.content:
                    print("Backend Error: will attempt to retry upload")
                    print(e)
                    return None
                else:
                    print("Unknown Error, please post an issue on github with this info")
                    print(e)
                    return None
            except retry_exceptions as e:
                print(f"A retriable error occurred: {e}")

            except Exception as e:
                if e in ACCEPTABLE_ERRNO:
                    print("Retriable Error occured, retrying now")
                else:
                    print(e)
                pass
            if response:
                if "id" in response:
                    print(f"Video link is https://www.youtube.com/watch?v={response['id']}")
                    return response['id']
                else:
                    print(response)
                    print(status)
                    return None


def get_youtube_service():
    """
    Gets a Credential object that can be used to access YouTube's API
    """
    CLIENT_SECRETS_FILE = get_secrets([
        os.path.expanduser("~"),
        sys.prefix,
        os.path.join(sys.prefix, "local"), "/usr",
        os.path.join("/usr", "local")
    ], ["share/frcuploader/client_secrets.json", "client_secrets.json"])

    flow = flow_from_clientsecrets(CLIENT_SECRETS_FILE, scope=YOUTUBE_UPLOAD_SCOPE)

    flow.user_agent = "FRC YouTube Uploader"
    storage = Storage(os.path.join(os.path.expanduser("~"), ".frc-oauth2-youtube.json"))
    credentials = storage.get()

    if credentials is None or credentials.invalid:
        credentials = run_flow(flow, storage)

    return build(
        YOUTUBE_API_SERVICE_NAME,
        YOUTUBE_API_VERSION,
        http=credentials.authorize(httplib2.Http()))


def get_spreadsheet_service():
    """
    Gets a Credential object that can be used to access Google Sheets' API
    """
    CLIENT_SECRETS_FILE = get_secrets([
        os.path.expanduser("~"),
        sys.prefix,
        os.path.join(sys.prefix, "local"), "/usr",
        os.path.join("/usr", "local")
    ], ["share/frcuploader/client_secrets.json", "client_secrets.json"])

    flow = flow_from_clientsecrets(CLIENT_SECRETS_FILE, scope=SPREADSHEETS_SCOPE)

    flow.user_agent = "FRC YouTube Uploader"

    storage = Storage(os.path.join(os.path.expanduser("~"), ".frc-oauth2-spreadsheet.json"))
    credentials = storage.get()

    if credentials is None or credentials.invalid:
        credentials = run_flow(flow, storage)

    http = credentials.authorize(httplib2.Http())
    discoveryUrl = ('https://sheets.googleapis.com/$discovery/rest?version=v4')
    return build('sheets', 'v4', http=http, discoveryServiceUrl=discoveryUrl)


def get_retry_status_codes():
    return RETRIABLE_STATUS_CODES


def get_retry_exceptions():
    return RETRIABLE_EXCEPTIONS


def get_max_retries():
    return 10


def get_secrets(prefixes, relative_paths):
    """
    Taken from https://github.com/tokland/youtube-upload/blob/master/youtube_upload/main.py
    Get the first existing filename of relative_path seeking on prefixes directories.
    """
    try:
        return os.path.join(sys._MEIPASS, relative_paths[-1])
    except Exception:
        for prefix in prefixes:
            for relative_path in relative_paths:
                path = os.path.join(prefix, relative_path)
                if os.path.exists(path):
                    return path



#!/usr/bin/env python3

import os
import re
import sys
import json
import time
import errno
import shutil
import random
import hashlib
import datetime as dt
from decimal import Decimal

import requests
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload

from .consts import *
from .youtube import upload

from cachecontrol import CacheControl
from cachecontrol.heuristics import ExpiresAfter

app_id = {'X-TBA-App-Id': ""}
trusted_auth = {'X-TBA-Auth-Id': "", 'X-TBA-Auth-Sig': ""}

s = requests.Session()
s = CacheControl(s, heuristic=ExpiresAfter(minutes=1))
s.headers.update(app_id)


"""Utility Functions"""


def convert_bytes(num):
    for x in sizes:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0


def file_size(path):
    file_info = os.stat(path)
    return convert_bytes(file_info.st_size)


def restart():
        args = sys.argv[:]

        args.insert(0, sys.executable)
        if sys.platform == 'win32':
            args = ['"%s"' % arg for arg in args]

        os.execv(sys.executable, args)


"""YouTube Title Generators"""


def quals_yt_title(options):
    return options.title if not options.replay else f"{options.title} Replay"


def quarters_yt_title(options):
    mnum = options.mnum
    if mnum <= 8:
        return f"{options.ename} - Quarterfinal Match {mnum}" if not options.replay else f"{options.ename} - Quarterfinal Match {mnum} Replay"
    elif mnum <= 12:
        mnum -= 8
        return f"{options.ename} - Quarterfinal Tiebreaker {mnum}" if not options.replay else f"{options.ename} - Quarterfinal Tiebreaker {mnum} Replay"
    else:
        raise ValueError("options.mnum must be within 1 and 12")


def semis_yt_title(options):
    mnum = options.mnum
    if mnum <= 4:
        return f"{options.ename} - Semifinal Match {mnum}" if not options.replay else f"{options.ename} - Semifinal Match {mnum} Replay"
    elif mnum <= 6:
        mnum -= 4
        return f"{options.ename} - Semifinal Tiebreaker {mnum}" if not options.replay else f"{options.ename} - Semifinal Tiebreaker {mnum} Replay"
    else:
        raise ValueError("options.mnum must be within 1 and 6")


def finals_yt_title(options):
    return f"{options.ename} - Final Match {options.mnum}" if not options.replay else f"{options.ename} - Final Match {options.mnum} Replay"


def ceremonies_yt_title(options):
    title = None
    if options.ceremonies is 1:
        if not options.eday:
            title = f"{options.ename} - {options.day} Opening Ceremonies"
        else:
            title = f"{options.ename} - Day {options.eday} Opening Ceremonies"
    elif options.ceremonies is 2:
        title = f"{options.ename} - Alliance Selection"
    elif options.ceremonies is 3:
        if not options.eday:
            title = f"{options.ename} - Closing Ceremonies"
        else:
            title = f"{options.ename} - Day {options.eday} Closing Ceremonies"
    elif options.ceremonies is 4:
        title = f"{options.ename} - Highlight Reel"
    return title


"""File Location Functions"""


def quals_filename(options):
    file = None
    for f in options.files:
        fl = f.lower()
        if all([" " + str(options.mnum) + "." in fl and any(k in fl for k in ("qual", "qualification", "qm"))]):
            if options.replay:
                if "replay" in fl:
                    file = f
                    break
            else:
                if "replay" not in fl:
                    file = f
                    break
    return file


def quarters_filename(options):
    file = None
    if 1 <= options.mnum <= 8:
        for f in options.files:
            fl = f.lower()
            if all(k in fl for k in ("quarter", "final", " " + str(options.mnum) + ".")):
                if "tiebreak" not in fl:
                    if options.replay:
                        if "replay" in fl:
                            file = f
                            break
                    else:
                        if "replay" not in fl:
                            file = f
                            break
    elif 9 <= options.mnum <= 12:
        mnum = options.mnum - 8
        for f in options.files:
            fl = f.lower()
            if all(k in fl for k in ("quarter", "tiebreak", "final", " " + str(mnum) + ".")):
                if options.replay:
                    if "replay" in fl:
                        file = f
                        break
                else:
                    if "replay" not in fl:
                        file = f
                        break
    return file


def semis_filename(options):
    file = None
    if options.mnum <= 4:
        for f in options.files:
            fl = f.lower()
            if all(k in fl for k in ("semi", "final", " " + str(options.mnum) + ".")):
                if "tiebreak" not in fl:
                    if options.replay:
                        if "replay" in fl:
                            file = f
                            break
                    else:
                        if "replay" not in fl:
                            file = f
                            break
    elif options.mnum <= 6:
        mnum = options.mnum - 4
        for f in options.files:
            fl = f.lower()
            if all(k in fl for k in ("semi", "tiebreak", "final", " " + str(mnum) + ".")):
                if options.replay:
                    if "replay" in fl:
                        file = f
                        break
                else:
                    if "replay" not in fl:
                        file = f
                        break
    return file


def finals_filename(options):
    file = None
    if options.mnum <= 2:
        for f in options.files:
            fl = f.lower()
            if all(k in fl for k in ("final", " " + str(options.mnum) + ".")):
                if all(k not in fl for k in ("quarter", "semi")) and "tiebreak" not in fl:
                    if options.replay:
                        if "replay" in fl:
                            file = f
                            break
                    else:
                        if "replay" not in fl:
                            file = f
                            break
    elif options.mnum >= 3:
        for f in options.files:
            fl = f.lower()
            if "final" in fl and any(k in fl for k in ("tiebreak", " " + str(options.mnum) + ".")):
                if all(k not in fl for k in ("quarter", "semi")):
                    if options.replay:
                        if "replay" in fl:
                            file = f
                            break
                    else:
                        if "replay" not in fl:
                            file = f
                            break
    return file


def ceremonies_filename(options):
    file = None
    if options.ceremonies is 1:
        for f in options.files:
            fl = f.lower()
            if all(k in fl for k in ("opening", "ceremon")):
                if any(k in fl for k in (options.day.lower(), f"day {options.eday}")):
                    file = f
                    break
    elif options.ceremonies is 2:
        for f in options.files:
            fl = f.lower()
            if all(k in fl for k in ("alliance", "selection")):
                file = f
                break
    elif options.ceremonies is 3:
        for f in options.files:
            fl = f.lower()
            if any(k in fl for k in ("closing", "award")) and "ceremon" in fl:
                if any(
                        k in fl for k in (options.day.lower(), f"day {options.eday}")):
                    file = f
                    break
                elif options.eday == 0:
                    file = f
                    break
    elif options.ceremonies is 4:
        for f in options.files:
            fl = f.lower()
            if any(k in fl for k in ("highlight", "wrapup", "recap")):
                file = f
                break
    return file


def create_names(options):
    if options.ceremonies is 0:
        fname = {
            "qm": quals_filename,
            "qf": quarters_filename,
            "sf": semis_filename,
            "f1m": finals_filename,
        }
        yt = {
            "qm": quals_yt_title,
            "qf": quarters_yt_title,
            "sf": semis_yt_title,
            "f1m": finals_yt_title,
        }
        try:
            if options.newest:
                return None, yt[options.mtype](options)
            else:
                return fname[options.mtype](options), yt[options.mtype](options)
        except KeyError:
            print(options.mtype)
    else:
        return ceremonies_filename(options), ceremonies_yt_title(options)


"""Match Code Generators"""


def quals_match_code(mtype, mnum):
    match_code = str(mtype) + str(mnum)
    return match_code


def quarters_match_code(mtype, mnum):
    match_set = str(mnum % 4)
    match_set = "4" if match_set == "0" else match_set
    match_code = mtype + match_set
    if mnum <= 4:
        match_code += "m1"
    elif mnum <= 8:
        match_code += "m2"
    elif mnum <= 12:
        match_code += "m3"
    else:
        raise ValueError("Match Number can't be larger than 12")
    return match_code


def semis_match_code(mtype, mnum):
    match_set = str(mnum % 2)
    match_set = "2" if match_set == "0" else match_set
    match_code = mtype + match_set
    if mnum <= 2:
        match_code += "m1"
    elif mnum <= 4:
        match_code += "m2"
    elif mnum <= 6:
        match_code += "m3"
    else:
        raise ValueError("Match Number can't be larger than 6")
    return match_code


def finals_match_code(mtype, mnum):
    match_code = mtype + str(mnum)
    return match_code


def get_match_code(mtype, mnum, mcode):
    if any(k == mcode for k in ("", "0")):
        switcher = {
            "qm": quals_match_code,
            "qf": quarters_match_code,
            "sf": semis_match_code,
            "f1m": finals_match_code,
        }
        return switcher[mtype](mtype, mnum)
    print(f"Uploading as {mcode}")
    return mcode.lower()


"""Data Compliation and Adjustment Functions"""


def get_match_results(event_key, match_key):
    match_data = tba.match("_".join([event_key, match_key]))
    if match_data is None:
        raise ValueError(f"{event_key} {match_key} does not exist on TBA. Please use a match that exists")
    blue_data, red_data = parse_data(match_data)
    while (blue_data[0] == -1 or red_data[0] == -1):
        print("Waiting 1 minute for TBA to update scores")
        time.sleep(60)
        match_data = tba.match("_".join([event_key, match_key]))
        blue_data, red_data = parse_data(match_data)
    return blue_data, red_data


def parse_data(match_data):
    blue = match_data['alliances']['blue']['team_keys']
    red = match_data['alliances']['red']['team_keys']
    blue_data = [match_data['alliances']['blue']['score']]
    red_data = [match_data['alliances']['red']['score']]
    for team in blue:
        blue_data.append(team[3:])
    for team in red:
        red_data.append(team[3:])
    return blue_data, red_data


def tba_results(options):
    mcode = get_match_code(options.mtype, options.mnum, options.mcode)
    blue_data, red_data = get_match_results(options.ecode, mcode)
    return blue_data, red_data, mcode


def create_description(options, blueScore, blue1, blue2, blue3, redScore, red1,
                       red2, red3):
    if all(x == -1 for x in (red1, red2, red3, redScore, blue1, blue2, blue3,
                             blueScore)):
        return NO_TBA_DESCRIPTION.format(
            ename=options.ename,
            team=options.prodteam,
            twit=options.twit,
            fb=options.fb,
            weblink=options.weblink)
    try:
        return options.description.format(
            ename=options.ename,
            team=options.prodteam,
            red1=red1,
            red2=red2,
            red3=red3,
            redscore=redScore,
            blue1=blue1,
            blue2=blue2,
            blue3=blue3,
            bluescore=blueScore,
            ecode=options.ecode,
            twit=options.twit,
            fb=options.fb,
            weblink=options.weblink)
    except TypeError as e:
        print(e)
        return options.description


def tiebreak_mnum(mnum, mtype):
    switcher = {
        "qm": mnum,
        "qf": mnum + 8,
        "sf": mnum + 4,
        "f1m": 3,
    }
    return switcher[mtype]


"""Additional YouTube Functions"""


def update_thumbnail(video_id, thumbnail):
    youtube.thumbnails().set(videoId=video_id, media_body=thumbnail).execute()
    print(f"Thumbnail added to video {video_id}")


def add_to_playlist(videoID, playlistID):
    if type(videoID) is list:  # Recursively add videos if videoID is list
        for vid in videoID:
            add_video_to_playlist(vid, playlistID)
    else:
        youtube.playlistItems().insert(
            part="snippet",
            body={
                'snippet': {
                    'playlistId': playlistID,
                    'resourceId': {
                        'kind': 'youtube#video',
                        'videoId': videoID
                    }
                }
            }).execute()
        print("Added to playlist")


"""TBA Trusted API"""


def post_video(token, secret, match_video, event_key, loc="match_videos"):
    trusted_auth = {'X-TBA-Auth-Id': "", 'X-TBA-Auth-Sig': ""}
    trusted_auth['X-TBA-Auth-Id'] = token
    request_path = f"/api/trusted/v1/event/{event_key}/{loc}/add"
    concat = secret + request_path + str(match_video)
    md5 = hashlib.md5(concat.encode("utf-8")).hexdigest()
    trusted_auth['X-TBA-Auth-Sig'] = str(md5)
    url_str = f"https://www.thebluealliance.com/api/trusted/v1/event/{event_key}/{loc}/add"
    if DEBUG:
        url_str = f"http://localhost:8080/api/trusted/v1/event/{event_key}/{loc}/add"
    if trusted_auth['X-TBA-Auth-Id'] == "" or trusted_auth['X-TBA-Auth-Sig'] == "":
        print("TBA ID and/or TBA secret missing. Please set them in the UI")
        return
    r = s.post(url_str, data=match_video, headers=trusted_auth)
    print(r.status_code)
    while 405 == r.status_code:
        print("Failed to POST to TBA")
        print("Attempting to POST to TBA again")
        r = s.post(url_str, data=match_video, headers=trusted_auth)
    if r.status_code > 299:
        print(r.text)
    elif "Success" in r.text or r.status_code == 200:
        print("Successfully added to TBA")
    else:
        print(r.text)
        print("Something went wrong")


def init(options):
    """The program starts here, options is a Namespace() object"""
    options.day = dt.datetime.now().strftime("%A")  # weekday in english ex: "Monday"
    options.files = list(reversed([f for f in os.listdir(options.where) if os.path.isfile(os.path.join(options.where, f)) and not f.startswith('.')]))
    options.tags = DEFAULT_TAGS.format(options.ecode, game=GAMES[options.ecode[:4]])  # add the ecode and game to default tags
    # default category is science & technology
    options.category = 28
    options.title = options.ename + f" - Qualification Match {options.mnum}"  # default title
    if any(k == options.description for k in ("Add alternate description here.", "", DEFAULT_DESCRIPTION)):
        options.description = DEFAULT_DESCRIPTION + CREDITS
    else:
        options.description += CREDITS

    # fix types except options.end
    options.ceremonies = int(options.ceremonies)
    options.mnum = int(options.mnum)
    options.eday = int(options.eday)

    # seperate case to push to TBA
    if options.ceremonies != 0:
        if options.tba:
            options.post = True
        else:
            options.post = False
        options.tba = False
    if options.tiebreak:
        options.mnum = tiebreak_mnum(options.mnum, options.mtype)
    if options.newest:
        x, options.yttitle = create_names(options)
    else:
        options.file, options.yttitle = create_names(options)

    if options.file is not None:
        print(f"Found {options.file} to upload")
        if options.end > options.mnum:
            upload_multiple_videos(options)
        else:
            try:
                print(pre_upload(options))
            except HttpError as e:
                print(f"An HTTP error {e.resp.status} occurred:\n{e.content}")
    else:
        print("No file found")
        return


def pre_upload(options):
    if not options.ceremonies:
        print(f"Initializing upload for {options.mtype} match {options.mnum}")
    else:
        print(f"Initializing upload for: {ceremonies_yt_title(options)}")
    print(f"YouTube title is {options.yttitle}")
    if len(options.yttitle) > 100:
        print(f"Title Length: {len(options.yttitle)} characters")
        return "Title cannot exceed 100 characters, please adjust your settings to reduce the title length"
    if options.tba:
        blue_data, red_data, mcode = tba_results(options)
        tags = options.tags.split(",")
        for team in blue_data[1:] + red_data[1:]:
            tags.append(f"frc{team}")
        tags.extend(options.ename.split(" "))
        tags.append("frc" + re.search('\D+', options.ecode).group())

        body = dict(
            snippet=dict(
                title=options.yttitle,
                description=create_description(options, *blue_data, *red_data),
                tags=tags,
                categoryId=options.category),
            status=dict(privacyStatus=options.privacy))
    else:
        mcode = get_match_code(options.mtype, options.mnum, options.mcode)

        tags = options.tags.split(",")
        tags.append("frc" + re.search('\D+', options.ecode).group())

        body = dict(
            snippet=dict(
                title=options.yttitle,
                description=create_description(options, -1, -1, -1, -1, -1, -1,
                                               -1, -1),
                tags=tags,
                categoryId=options.category),
            status=dict(privacyStatus=options.privacy))
    if options.newest:
        options.vid = upload(youtube, body, options.file)
    else:
        options.vid = upload(youtube, body, os.path.join(options.where, options.file))
    return post_upload(options, mcode)


def post_upload(options, mcode):
    try:
        if "thumbnail.png" in options.files:
            update_thumbnail(options.vid, os.path.join(options.where, "thumbnail.png"))
        else:
            print("thumbnail.png does not exist")
    except HttpError as e:
        if e.resp.status in retry_status_codes:
            error = f"A retriable HTTP error {e.resp.status} occurred:\n{e.content}"
    except retry_exceptions as e:
        error = f"A retriable error occurred: {e}"

    try:
        add_to_playlist(options.vid, options.pID)
    except Exception as e:
        print(e)
        print("Failed to post to playlist")

    if options.tba:
        request_body = json.dumps({mcode: options.vid})
        post_video(options.tbaID, options.tbaSecret, request_body,
                   options.ecode)
    elif options.ceremonies and options.post:
        request_body = json.dumps([options.vid])
        post_video(options.tbaID, options.tbaSecret, request_body,
                   options.ecode, "media")

    wasBatch = "True" if options.end else "False"
    usedTBA = "True" if options.tba else "False"
    totalTime = dt.datetime.now() - options.then
    values = [[
        str(dt.datetime.now()),
        str(totalTime), f"https://www.youtube.com/watch?v={options.vid}", usedTBA, options.ename, wasBatch, mcode
    ]]
    sheetbody = {'values': values}
    try:
        spreadsheet.spreadsheets().values().append(
            spreadsheetId=spreadsheetID,
            range=rowRange,
            valueInputOption="USER_ENTERED",
            body=sheetbody).execute()
        print("Added data to spreadsheet")
    except Exception as e:
        print("Failed to write to spreadsheet")

    if options.sendto:
        if options.newest:
            print("Moving the file with Get Newest File is not supported yet")
        else:
            try:
                print("Moving file")
                shutil.move(os.path.join(options.where, options.file), os.path.join(options.sendto, options.file))
            except Exception as e:
                print(f"Could not copy to {options.sendto}")
                print(e)
    return f"DONE UPLOADING {options.file}\n"
